dofile_once("data/scripts/lib/utilities.lua")
dofile_once("data/scripts/lib/coroutines.lua")
dofile_once("data/scripts/streaming_integration/event_utilities.lua")

STREAMING_EVENT_AWFUL = 0
STREAMING_EVENT_BAD = 1
STREAMING_EVENT_NEUTRAL = 2
STREAMING_EVENT_GOOD = 3
STREAMING_EVENT_AUTHOR_NOLLAGAMES = "Nolla Games"

function spawn_object(which_one, player_id)
	local x,y = EntityGetTransform( player_id )
	
	-- local opts = { "portal_orb_blue", "portal_orb_green", "portal_orb_pink" }
	
	SetRandomSeed( GameGetFrameNum(), x + y + player_id )
	
	local raycasts = 4
	local dir = ( math.pi * 2.0 ) / raycasts
	local length = 8
	
	-- local rnd = Random( 1, #opts )
	local portal = "mods/pinball/files/twitch/" .. which_one .. ".xml"
	
	local p = EntityGetWithTag( "player_unit" )
	
	if ( #p > 0 ) then
		local t = player_id
		local px, py = EntityGetTransform( t )
		
		local dx = px + Random( -150, 150 )
		local dy = py + Random( -150, 150 )
		
		local dist = math.abs( py - dy ) + math.abs( px - dx )
		local limit = 0
		local wall = true
		
		while ( ( dist < 100 ) or wall ) and ( limit < 20 ) do
			dx = px + Random( -100, 100 )
			dy = py + Random( -100, 100 )
			dist = math.abs( py - dy ) + math.abs( px - dx )
			limit = limit + 1
			
			for j=0,raycasts-1 do
				local ex = dx + math.cos( j * dir ) * length
				local ey = dy - math.sin( j * dir ) * length
				
				wall = RaytraceSurfaces( dx, dy, ex, ey )
				
				if wall then
					break
				end
			end
		end
		if portal == "mods/pinball/files/twitch/bad_thing" then
			EntityLoad( "mods/pinball/files/death_sparks.xml", dx, dy )
		end
		local portal_id = EntityLoad( portal, dx, dy )
	end
end

streaming_events =
{
	{
		id = "BAD_THING",
		ui_name = "$pinball_bad_thing",
		ui_description = "$pinball_bad_thing_desc",
		ui_icon = "data/ui_gfx/streaming_event_icons/health_plus.png",
		ui_author = STREAMING_EVENT_AUTHOR_NOLLAGAMES,
		weight = 1.0,
		kind = STREAMING_EVENT_BAD,
		action = function(event)
			for i,entity_id in ipairs( get_players() ) do
				spawn_object("bad_thing", entity_id)
			end
		end,
	},
	{
		id = "GRAVITY_FIELD",
		ui_name = "$pinball_gravity_field",
		ui_description = "$pinball_gravity_field_desc",
		ui_icon = "data/ui_gfx/streaming_event_icons/health_plus.png",
		ui_author = STREAMING_EVENT_AUTHOR_NOLLAGAMES,
		weight = 1.0,
		kind = STREAMING_EVENT_BAD,
		action = function(event)
			for i,entity_id in ipairs( get_players() ) do
				spawn_object("gravity_field", entity_id)
			end
		end,
	},
	{
		id = "BUMPER",
		ui_name = "$pinball_bumper",
		ui_description = "$pinball_bumper_desc",
		ui_icon = "data/ui_gfx/streaming_event_icons/health_plus.png",
		ui_author = STREAMING_EVENT_AUTHOR_NOLLAGAMES,
		weight = 1.0,
		kind = STREAMING_EVENT_BAD,
		action = function(event)
			for i,entity_id in ipairs( get_players() ) do
				spawn_object("bumper_strong", entity_id)
			end
		end,
	},
	{
		id = "BUMPER_EXPLOSIVE",
		ui_name = "$pinball_bumper_explosive",
		ui_description = "$pinball_bumper_explosive_desc",
		ui_icon = "data/ui_gfx/streaming_event_icons/health_plus.png",
		ui_author = STREAMING_EVENT_AUTHOR_NOLLAGAMES,
		weight = 1.0,
		kind = STREAMING_EVENT_BAD,
		action = function(event)
			for i,entity_id in ipairs( get_players() ) do
				spawn_object("bumper_explosive", entity_id)
			end
		end,
	},
	{
		id = "VINE_MONSTER",
		ui_name = "$pinball_vine_monster",
		ui_description = "$pinball_vine_monster_desc",
		ui_icon = "data/ui_gfx/streaming_event_icons/health_plus.png",
		ui_author = STREAMING_EVENT_AUTHOR_NOLLAGAMES,
		weight = 1.0,
		kind = STREAMING_EVENT_BAD,
		action = function(event)
			for i,entity_id in ipairs( get_players() ) do
				spawn_object("vine_monster", entity_id)
			end
		end,
	},
	{
		id = "SUPER_SPEED",
		ui_name = "$pinball_super_speed",
		ui_description = "$pinball_super_speed_desc",
		ui_icon = "data/ui_gfx/streaming_event_icons/speedy_enemies.png",
		ui_author = STREAMING_EVENT_AUTHOR_NOLLAGAMES,
		weight = 1.0,
		kind = STREAMING_EVENT_BAD,
		action = function(event)
			for i,entity_id in ipairs( get_players() ) do			
				local lua_comps = EntityGetComponent(entity_id, "LuaComponent") or {}
				
				for i, comp in ipairs(lua_comps) do
				
					if ComponentGetValue2(comp, "script_source_file") == "mods/pinball/files/twitch/scripts/speed_toggler_off.lua" then
					
						component1 = true
						
					end
					
				end
				
				if not component1 then
					local vel_comp = EntityGetFirstComponentIncludingDisabled( entity_id, "VelocityComponent" )
					ComponentSetValue2( vel_comp, "terminal_velocity", 600)

					EntityAddComponent(entity_id, "LuaComponent", {
						script_source_file="mods/pinball/files/twitch/scripts/speed_toggler_off.lua",
						execute_every_n_frame=600,
						remove_after_executed=1,
						execute_on_added=0,
					})
				end
			end
		end,
	},
	
	{
		id = "NO_GRAVITY",
		ui_name = "$pinball_no_gravity",
		ui_description = "$pinball_no_gravity_desc",
		ui_icon = "data/ui_gfx/streaming_event_icons/speedy_enemies.png",
		ui_author = STREAMING_EVENT_AUTHOR_NOLLAGAMES,
		weight = 1.0,
		kind = STREAMING_EVENT_NEUTRAL,
		action = function(event)
		
			for i,entity_id in ipairs( get_players() ) do		
			
				local lua_comps = EntityGetComponent(entity_id, "LuaComponent") or {}
				
				for i, comp in ipairs(lua_comps) do
				
					if ComponentGetValue2(comp, "script_source_file") == "mods/pinball/files/twitch/scripts/gravity_toggler_off.lua" then
					
						component2 = true
						
					end
					
				end
				
				if not component2 then
				
					local vel_comp = EntityGetFirstComponentIncludingDisabled( entity_id, "VelocityComponent" )
					ComponentSetValue2( vel_comp, "gravity_y", 0)

					EntityAddComponent(entity_id, "LuaComponent", {
						script_source_file="mods/pinball/files/twitch/scripts/gravity_toggler_off.lua",
						execute_every_n_frame=1200,
						remove_after_executed=1,
						execute_on_added=0,
					})
					
				end
			end
		end,
	},
	
	{
		id = "INVERTED_GRAVITY",
		ui_name = "$pinball_invert_gravity",
		ui_description = "$pinball_invert_gravity_desc",
		ui_icon = "data/ui_gfx/streaming_event_icons/speedy_enemies.png",
		ui_author = STREAMING_EVENT_AUTHOR_NOLLAGAMES,
		weight = 1.0,
		kind = STREAMING_EVENT_BAD,
		action = function(event)
			for i,entity_id in ipairs( get_players() ) do		
			
				local lua_comps = EntityGetComponent(entity_id, "LuaComponent") or {}
				
				for i, comp in ipairs(lua_comps) do
				
					if ComponentGetValue2(comp, "script_source_file") == "mods/pinball/files/twitch/scripts/gravity_toggler_off.lua" then
					
						component3 = true
						
					end
					
				end
				
				if not component3 then
					local vel_comp = EntityGetFirstComponentIncludingDisabled( entity_id, "VelocityComponent" )
					ComponentSetValue2( vel_comp, "gravity_y", -250)

					EntityAddComponent(entity_id, "LuaComponent", {
						script_source_file="mods/pinball/files/twitch/scripts/gravity_toggler_off.lua",
						execute_every_n_frame=600,
						remove_after_executed=1,
						execute_on_added=0,
					})
				end
			end
		end,
	},
}