dofile_once( "data/scripts/lib/utilities.lua" )

local cooldown_frames = GetValueInteger("cooldown_frames", 0)
if cooldown_frames > 0 then
	SetValueInteger("cooldown_frames", cooldown_frames - 1)
end

--Bumper part

local entity_id = GetUpdatedEntityID()

local x, y = EntityGetTransform( entity_id )

SetRandomSeed( GameGetFrameNum(), x + y + entity_id )

local projectiles = EntityGetWithTag( "player_unit" )

local direction_random = 0
if ( ( #projectiles > 0 ) and ( GlobalsGetValue("pinball_respawn_in_progress", "0") == "0" ) ) and cooldown_frames <= 0 then
	for i,projectile_id in ipairs(projectiles) do	
		local px, py = EntityGetTransform( projectile_id )
		-- EntitySetComponentsWithTagEnabled(projectile_id, "start", true)
		local distance = math.abs( x - px ) + math.abs( y - py )
		local distance_full = 37
		
		if ( distance < distance_full * 1.25 ) then
			distance = get_distance( px, py, x, y )
			direction = get_direction( px, py, x, y )
	
			if ( distance < distance_full ) then
				local value = GlobalsGetValue("pinball_boss_start", "0")
				GlobalsSetValue("pinball_boss_start", "1")
				if value == "0" and GlobalsGetValue("pinball_boss_start", "0") == "1" then
					fight_player = true

					if( fight_player ) then
		
						EntitySetComponentsWithTagEnabled( entity_id, "disabled_at_start", true )
						EntitySetComponentsWithTagEnabled( entity_id, "enabled_at_start", false )
						PhysicsSetStatic( entity_id, false )

						if EntityHasTag( entity_id, "boss_centipede" ) then
							EntityAddTag( entity_id, "boss_centipede_active" )
							
							local child_entities = EntityGetAllChildren( entity_id )
							local child_to_remove = 0
							
							if ( child_entities ~= nil ) then
								for i,child_id in ipairs( child_entities ) do
									EntityHasTag( child_id, "protection" )
									child_to_remove = child_id
								end
							end
							
							if ( child_to_remove ~= 0 ) then
								EntityKill( child_to_remove )
							end
						end
					end
				end
				
				local velocitycomponents = EntityGetComponent( projectile_id, "VelocityComponent" )
				
				local gravity_percent = math.max(( distance_full - distance ) / distance_full, 0.01)
				local gravity_coeff = 1000000000000000
				
				if ( velocitycomponents ~= nil ) then
					edit_component( projectile_id, "VelocityComponent", function(comp,vars)
						local vel_x,vel_y = ComponentGetValueVector2( comp, "mVelocity", vel_x, vel_y)
						
						local offset_x = math.cos( direction + direction_random ) * ( gravity_coeff * gravity_percent )
						local offset_y = 0 - math.sin( direction + direction_random ) * ( gravity_coeff * gravity_percent )

						vel_x = vel_x + offset_x
						vel_y = vel_y + offset_y
						
						local lua_comps = EntityGetComponent(projectile_id, "LuaComponent") or {}
						
						for i, lua_comp in ipairs(lua_comps) do
						
							if ComponentGetValue2(lua_comp, "script_source_file") ~= "mods/pinball/files/scripts/speed_toggler_off.lua" then

								ComponentSetValue2( comp, "terminal_velocity", 800)

								EntityAddComponent(projectile_id, "LuaComponent", {
									script_source_file="mods/pinball/files/scripts/speed_toggler_off.lua",
									execute_every_n_frame=40,
									execute_on_added=0,
								})
								
							end
							
						end
						
						ComponentSetValueVector2( comp, "mVelocity", vel_x, vel_y)
					end)
					EntityLoad("mods/pinball/files/bounce_sparks_explosive.xml", px, py)
					
					SetValueInteger("cooldown_frames", 30)
				end
			end
		end
	end
end

local player_in_range = EntityGetInRadiusWithTag(x, y, 1300, "player_unit")

--Adding health bar
if GlobalsGetValue("pinball_boss_start", "0") == "1" and #player_in_range > 0 then
	gui = gui or GuiCreate()
	GuiStartFrame(gui)
	local screen_width, screen_height = GuiGetScreenDimensions(gui)
	local healthbar_width = (87 / 2)
	local healthbar_height = 8
	local healthbar_offset_y = screen_height * 0.213889 - 43
	local hp_max = 5
	local hp = tonumber(GlobalsGetValue("pinball_player_health", "5")) -- SET YOUR HP HEEEEEEEEEEEEEREEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
	if hp < 0 then
	  return
	end

	local percent_filled = hp / hp_max
	local inner_healthbar_width = (healthbar_width - 2) * percent_filled
	-- Outline / Background
	GuiOptionsAddForNextWidget(gui, 2) -- NonInteractive
	GuiColorSetForNextWidget(gui, 0.412, 0.243, 0.200, 1)
	GuiZSetForNextWidget(gui, -100)
	GuiImage(gui, 2, (screen_width - healthbar_width) / 2, screen_height - healthbar_offset_y, "data/debug/whitebox.png", 1, healthbar_width / 20, healthbar_height / 20) -- 20x20
	-- Actual inner bar
	GuiOptionsAddForNextWidget(gui, 2) -- NonInteractive
	GuiColorSetForNextWidget(gui, 0.529, 0.749, 0.110, 1)
	GuiZSetForNextWidget(gui, -101)
	GuiImage(gui, 3, (screen_width - healthbar_width) / 2 + 1, screen_height - healthbar_offset_y + 1, "data/debug/whitebox.png", 1, inner_healthbar_width / 20, (healthbar_height - 2) / 20) -- 20x20
	-- Health icon
	GuiOptionsAddForNextWidget(gui, 2) -- NonInteractive
	GuiZSetForNextWidget(gui, -102)
	GuiImage(gui, 4, (screen_width - healthbar_width) / 2 + healthbar_width + 2.5, screen_height - healthbar_offset_y - 1, "data/ui_gfx/hud/health.png", 1, 1, 1) -- 20x20
	-- Text
	local text_width = GuiGetTextDimensions(gui, hp, 1)
	GuiColorSetForNextWidget(gui, 0, 0, 0, 1)
	GuiZSetForNextWidget(gui, -103)
	GuiText(gui, (screen_width - text_width) / 2, screen_height - healthbar_offset_y - 1, hp)
	-- "Player" string
	local player_string = GameTextGetTranslatedOrNot("$animal_player")
	local text_width = GuiGetTextDimensions(gui, player_string, 1)
	GuiColorSetForNextWidget(gui, 0.8, 0.8, 0.8, 1)
	GuiZSetForNextWidget(gui, -106)
	GuiText(gui, (screen_width - text_width) / 2, screen_height - healthbar_offset_y - 13, player_string)
end