local function get_direction( x1, y1, x2, y2 )
	return math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
end

local function get_distance( x1, y1, x2, y2 )
	return math.sqrt( ( x2 - x1 ) ^ 2 + ( y2 - y1 ) ^ 2 )
end

local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)
local prev_x, prev_y = GetValueNumber("prev_x", x), GetValueNumber("prev_y", y)
local distance_full = 11

local number = GetValueNumber("number", 0)
SetValueNumber("number", number + 1)

-- Returns true if we bumped
local function try_bump(entity_id, x, y)
	local other_entity_x, other_entity_y = EntityGetTransform(entity_id)
	local distance = get_distance(other_entity_x, other_entity_y, x, y)
	if distance < distance_full then
		local vel_comp = EntityGetFirstComponentIncludingDisabled(entity_id, "VelocityComponent")
		if vel_comp then
			local gravity_percent = math.max(( distance_full - distance ) / distance_full, 0.01)
			local gravity_coeff = 1000000
			local vel_x, vel_y = ComponentGetValueVector2(vel_comp, "mVelocity")
			direction = get_direction(other_entity_x, other_entity_y, x, y)
			local offset_x = math.cos(direction) * gravity_coeff * gravity_percent
			local offset_y = math.sin(direction) * gravity_coeff * gravity_percent
			if number > 30 then
				if ModIsEnabled("noita-together") then
					if GameHasFlagRun("NT_unlocked_controls") then
						vel_x = vel_x - offset_x
						vel_y = vel_y - offset_y
						ComponentSetValue2(vel_comp, "mVelocity", vel_x * 0.01, vel_y * 0.01)
						PhysicsApplyForce(entity_id, vel_x * 0.02, vel_y * 0.02)
						EntityLoad("mods/pinball/files/bounce_sparks.xml", other_entity_x, other_entity_y)
						return true
					end
				else
					vel_x = vel_x - offset_x
					vel_y = vel_y - offset_y
					ComponentSetValue2(vel_comp, "mVelocity", vel_x * 0.01, vel_y * 0.01)
					PhysicsApplyForce(entity_id, vel_x * 0.02, vel_y * 0.02)
					EntityLoad("mods/pinball/files/bounce_sparks.xml", other_entity_x, other_entity_y)
					return true
				end
			end
		end
	end
end

if GlobalsGetValue("pinball_respawn_in_progress", "0") == "0" then
  for i, entity_id in ipairs(EntityGetInRadiusWithTag(x, y, 500, "projectile") or {}) do
    local dir = get_direction(prev_x, prev_y, x, y)
    local move_distance = get_distance(prev_x, prev_y, x, y)
    local dir_x, dir_y = math.cos(dir), math.sin(dir)
    local vx, vy = x - prev_x, y - prev_y
    prev_x = prev_x + vx
    prev_y = prev_y + vy
    local move_per_step = 3
    local max_tries = 50
    local current_try = 0
    repeat
      current_try = current_try + 1
      move_distance = move_distance - move_per_step
      move_distance = math.max(0, move_distance)
      prev_x = prev_x + dir_x * math.min(move_per_step, move_distance)
      prev_y = prev_y + dir_y * math.min(move_per_step, move_distance)
      if try_bump(entity_id, prev_x, prev_y) then
        break
      end
    until move_distance < 1 or current_try >= max_tries
  end
end

SetValueNumber("prev_x", x)
SetValueNumber("prev_y", y)